import requests
import zipfile
from io import BytesIO
import os

BASE_URL = 'https://api.valyu.network/v1'

def load_dataset_samples(dataset_identifier: str, save_path: str = 'downloads') -> bool:
    """
    Retrieve and download sample data for a dataset.

    Args:
        dataset_identifier (str): A string in the format 'org_id/dataset_name'.
            For example: 'Valyu/papers'
        save_path (str, optional): Directory to save the downloaded samples. 
            Defaults to 'downloads'.

    Raises:
        ValueError: If the dataset_identifier is not in the correct format.
        requests.exceptions.RequestException: If there's an error fetching the data.
    """
    try:
        print(f"Fetching samples...")
        
        # Validate and unpack the dataset identifier
        if '/' not in dataset_identifier:
            raise ValueError("dataset_identifier must be in the format 'org_id/dataset_name'")
        org_id, dataset_name = dataset_identifier.split('/', 1)
        
        # Construct the API request
        params = {
            "orgId": org_id,
            "datasetName": dataset_name
        }
        
        # Fetch the presigned URL for samples
        response = requests.get(f"{BASE_URL}/training/samples", params=params)
        response.raise_for_status()
        data = response.json()
        presigned_url = data['presigned_url']
        
        # Download the zip file
        try:
            response = requests.get(presigned_url)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            print(f"Error downloading the zip file: {e}")
            return False
        
        # Extract the contents
        with zipfile.ZipFile(BytesIO(response.content)) as zip_ref:
            for file_info in zip_ref.infolist():
                if not file_info.filename.startswith('__MACOSX'):
                    zip_ref.extract(file_info, save_path)
        
        # Remove any empty directories
        for root, dirs, files in os.walk(save_path, topdown=False):
            for dir in dirs:
                dir_path = os.path.join(root, dir)
                if not os.listdir(dir_path):
                    os.rmdir(dir_path)
        
        print(f"Sample data downloaded and extracted successfully to {save_path}.")
    except requests.exceptions.RequestException as e:
        print(f"Error fetching sample data: {e}")
    except Exception as e:
        print(f"An error occurred: {e}")



from typing import List, Optional, Literal
from pydantic import BaseModel

SearchType = Literal['web', 'proprietary']

class SearchResult(BaseModel):
    title: str
    url: str
    content: str
    description: Optional[str] = None
    source: str
    price: float
    length: int
    org_id: str
    relevance_score: float

class ResultsBySource(BaseModel):
    web: int
    proprietary: int

class SearchResponse(BaseModel):
    success: bool
    error: Optional[str] = None
    query: str
    results: List[SearchResult]
    results_by_source: ResultsBySource
    total_deduction_pcm: float
    total_deduction_dollars: float
    total_characters: int

    def __str__(self) -> str:
        return self.model_dump_json(indent=2)


import requests
import os
import time
import zipfile
from concurrent.futures import ThreadPoolExecutor
from tqdm import tqdm

BASE_URL = 'https://api.valyu.network/v1'

def load_dataset(api_key, dataset_id, save_path='downloads'):
    try:
        """Retrieve and download data from the intermediary service to a specified file path."""
        
        print(f"Fetching dataset {dataset_id}...")

        # Transform dataset_id from orgId/datasetId to orgId:datasetId format and URL encode
        org_id, dataset_name = dataset_id.split('/')
        encoded_dataset_id = f"{org_id}%3A{dataset_name}"

        headers = {'x-api-key': api_key}
    
        # Fetch the manifest URL with the transformed dataset ID
        response = requests.get(f"{BASE_URL}/training/datasets/{encoded_dataset_id}", headers=headers)
        response.raise_for_status()

        data = response.json()
        manifest_url = data['presignedURL']
    
        # Download the manifest file
        response = requests.get(manifest_url)
        response.raise_for_status()
        manifest = response.json()

        # Create the save directory if it doesn't exist
        os.makedirs(save_path, exist_ok=True)

        # Function to download a single file
        def download_file(file_info):
            # Skip directories
            if file_info['key'].endswith('/'):
                return
            
            file_url = file_info['presignedUrl']
            file_path = os.path.join(save_path, file_info['key'])
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            file_response = requests.get(file_url)
            with open(file_path, 'wb') as f:
                f.write(file_response.content)

        # Download files concurrently
        with ThreadPoolExecutor() as executor:
            list(tqdm(executor.map(download_file, manifest), total=len(manifest), desc="Downloading files"))

        print(f"\nDataset downloaded successfully into {save_path}.")
    
    except Exception as e:
        raise Exception(f"Error occurred: {e}")



import json
import requests
from pydantic import BaseModel
from typing import Optional, List
from valyu.types.context import SearchResponse, SearchType, ResultsBySource
import os

class ErrorResponse(BaseModel):
    success: bool
    error: str

class Valyu:
    def __init__(
        self, 
        api_key: Optional[str],
        base_url: str = "https://api.valyu.network/v1"
    ):
        """
        Initialize the Valyu client.

        Args:
            api_key (Optional[str]): The API key to use for the client.
            base_url (str): The base URL for the Valyu API.
        """
        if not api_key:
            api_key = os.getenv("VALYU_API_KEY")
            if not api_key:
                raise ValueError("VALYU_API_KEY is not set")

        self.base_url = base_url
        self.headers = {
            "Content-Type": "application/json",
            "x-api-key": api_key
        }

    def context(
        self, 
        query: str, 
        search_type: SearchType, 
        num_query: int = 10,
        num_results: int = 10,
        max_price: int = 1,
        data_sources: Optional[List[str]] = None
    ) -> Optional[SearchResponse]:
        """
        Fetch context from the Valyu API.

        Args:
            query (str): The query to search for.
            search_type (SearchType): The type of search to perform.
            num_query (int): The number of queries to run.
            num_results (int): The number of results to return per query.
            max_price (int): The maximum price (per thousand queries) to spend on the search. 

        Returns:
            Optional[SearchResponse]: The search response.
        """
        try:
            payload = {
                "query": query,
                "search_type": search_type,
                "num_query": num_query,
                "num_results": num_results,
                "max_price": max_price
            }

            if data_sources is not None:
                payload["data_sources"] = data_sources

            response = requests.post(
                f"{self.base_url}/knowledge", 
                json=payload,
                headers=self.headers
            )

            data = response.json()
            if not response.ok:
                # Create a SearchResponse with error information
                return SearchResponse(
                    success=False,
                    error=data.get("error"),
                    query=query,
                    results=[],
                    results_by_source=ResultsBySource(web=0, proprietary=0),
                    total_deduction_pcm=0.0,
                    total_deduction_dollars=0.0,
                    total_characters=0
                )

            return SearchResponse(**data)
        except Exception as e:
            # Create a SearchResponse with the exception information
            return SearchResponse(
                success=False,
                error=str(e),
                query=query,
                results=[],
                results_by_source=ResultsBySource(web=0, proprietary=0),
                total_deduction_pcm=0.0,
                total_deduction_dollars=0.0,
                total_characters=0
            )



# setup.py

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='valyu',
    version='1.0.0',
    author='Valyu',
    author_email='contact@valyu.network',
    maintainer='Harvey Yorke',
    maintainer_email='harvey@valyu.network',
    description='We handle the data. You handle the AI.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://valyu.network",
    packages=find_packages(exclude=['tests*', '*.downloads*']),
    package_data={
        'valyu': ['py.typed'],
    },
    exclude_package_data={
        '': ['*.pyc', '*.pyo', '*.pyd', '__pycache__', '*.so'],
    },
    install_requires=[
        'requests>=2.31.0',
        'pydantic>=2.5.2',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
